@@ -2,11 +2,10 @@ use clap::Parser;
 use std::fs::{metadata, read_dir};
 use std::path::Path;
 use std::time::Instant;
-use indicatif::{ProgressStyle, ProgressBar, MultiProgress};
 use console::style;
 
 mod types;
-use types::{DirMap, Group, GroupList};
+use types::{DirMap, Group, GroupList, ProgressOptional};
 
 mod display_u64_as_file_size;
 use display_u64_as_file_size::DisplayFileSize;
@@ -17,7 +16,10 @@ use display_duration_as_hms::Hms;
 mod display_letters_by_u8;
 use display_letters_by_u8::{count_to_letter, produce_letter};
 
-static DEFAULT_PRINT_LIMMIT: u8 = 25;
+mod progress_bar;
+use progress_bar::{init_progress_bar, new_progress, tick_progress};
+
+static DEFAULT_PRINT_LIMMIT: u32 = 25;
 static DEFAULT_PRINT_LIMMIT_STR: &str = "25";
 
 /// Size of directory optic
@@ -38,36 +40,39 @@ struct Args {
     /// Sort the parent directories
     #[clap(short,long)]
     sort: bool,
-    /// Sort and limit the parent directories
+    /// Sort and limit the count of parent directories
     #[clap(short,long,default_value = DEFAULT_PRINT_LIMMIT_STR)]
-    limit: u8,
+    limit: u32,
+    /// Print and limit the count of children directories
+    #[clap(long,default_value = "9999")]
+    child_limit: u32,
     /// Print all the parent directories, no limit
     #[clap(short,long)]
     all: bool,
     /// Print child of parent directories
     #[clap(short,long)]
     map: bool,
+    /// Doesnt show progress which causes better performance
+    #[clap(short,long)]
+    fast: bool,
 }
 
 fn main() {
     let args = Args::parse();
     let path = Path::new(&args.dir);
-    let progress_style = ProgressStyle::with_template(
-        "{prefix:.bold.dim} {spinner}   {wide_msg}"
-    ).unwrap()
-        .tick_chars("⠁⠁⠂⠂⠄⠄⡀⡀⢀⢀⠠⠠⠐⠐⠈⠈");
-    let progress_steam = MultiProgress::new();
     let mut dirs: GroupList = Vec::new();
     let mut count = 0u64;
     let mut size = 0u64;
 
+    let progress = init_progress_bar(args.fast);
+
     println!("\n\tSize of {}", args.dir);
 
     let start_runtime = Instant::now();
-    dir_size(path,&mut size, &mut count, &mut dirs, &progress_steam,  &progress_style);
+    dir_size(path,&mut size, &mut count, &mut dirs, &progress);
     let runtime = start_runtime.elapsed();
 
-    if args.print || args.sort || args.limit != DEFAULT_PRINT_LIMMIT {
+    if args.print || args.sort || args.map || args.limit != DEFAULT_PRINT_LIMMIT {
         print_dirs(&mut dirs, &size, &args);
     }
 
@@ -80,13 +85,13 @@ fn main() {
     );
 }
 
+
 fn dir_size(
     path: &Path,
     size: &mut u64,
     count: &mut u64,
     dirs: &mut GroupList,
-    progress_stream: &MultiProgress,
-    progress_style: &ProgressStyle
+    progress: &ProgressOptional
 ) {
     if !path.exists() { return }
     if path.is_file() {
@@ -97,29 +102,21 @@ fn dir_size(
         return
     }
     if path.is_dir() {
-        let pb = progress_stream.add(ProgressBar::new_spinner());
-        pb.set_style(progress_style.clone());
-        pb.set_prefix(format!("[{}]", size.display_as_file_size()));
         let mut children = Vec::new();
         let mut path_size = 0u64;
 
+        let pb = new_progress(progress, &size.display_as_file_size());
+
         for child in read_dir(path).unwrap() {
             let child = child.unwrap();
             let child_path = child.path();
-            pb.set_message(
-                child_path.to_owned()
-                    .into_os_string()
-                    .into_string()
-                    .unwrap()
-            );
-            pb.inc(1);
+            tick_progress(&pb, &child_path);
             dir_size(
                 &child_path,
                 &mut path_size,
                 count,
                 &mut children,
-                progress_stream,
-                progress_style
+                progress
             );
         }
         *size += path_size;
@@ -138,19 +135,15 @@ fn print_dirs(dirs: &mut GroupList, size: &u64, args: &Args) {
     if args.sort || args.map {
         dirs.sort_by(|a, b| b.parent.size.cmp(&a.parent.size))
     }
-    if args.all {
-        println!("");
-        return
-    }
 
     let space_count = 10u8;
     println!(
         "SIZE   {}SUBS\tDIRECTORY",
         produce_letter(space_count, 4, ' ')
     );
-    let mut index = 0;
+    let mut index = 0u32;
     for group in dirs.iter() {
-        if index >= args.limit {
+        if !args.all && index >= args.limit {
             break
         }
         index += 1;
@@ -164,7 +157,7 @@ fn print_dir_children(
     space_count: u8,
     max_size: &u64,
     generation: u8,
-    index: &mut u8,
+    index: &mut u32,
     args: &Args
 ) {
     let mut children = group.children.clone();
@@ -173,8 +166,9 @@ fn print_dir_children(
         children.sort_by(|a, b| b.parent.size.cmp(&a.parent.size))
     }
 
+    let mut children_index = 0u32;
     for child in children {
-        if *index >= args.limit {
+        if !args.all && *index >= args.limit || children_index >= args.child_limit {
             println!(
                 "\t{}... other child dirs are included",
                 count_to_letter(2 * generation, ' '),
@@ -186,12 +180,13 @@ fn print_dir_children(
         let children_count = child.children.len();
         let children_count_string_len = format!("{}", children_count).len() as u8;
         *index += 1;
+        children_index += 1;
         let styled_size = match 100 * child.parent.size / *max_size {
-            0..=20      => style(child.parent.size.display_as_file_size()),
-            21..=40     => style(child.parent.size.display_as_file_size()).green(),
-            41..=60     => style(child.parent.size.display_as_file_size()).blue(),
-            61..=80     => style(child.parent.size.display_as_file_size()).yellow(),
-            81..=100    => style(child.parent.size.display_as_file_size()).red(),
+            0..=5      => style(child.parent.size.display_as_file_size()),
+            6..=10     => style(child.parent.size.display_as_file_size()).green(),
+            11..=15     => style(child.parent.size.display_as_file_size()).blue(),
+            16..=20     => style(child.parent.size.display_as_file_size()).yellow(),
+            21..=100    => style(child.parent.size.display_as_file_size()).red(),
             _           => style(child.parent.size.display_as_file_size())
         };
         println!(
@@ -203,7 +198,7 @@ fn print_dir_children(
             count_to_letter(2 * generation, '-'),
             child.parent.dirname,
         );
-        if args.map {
+        if args.map && child.children.len() > 0 {
             print_dir_children(&child, space_count, max_size, generation + 1, index, args);
         }
     };
